#include "AxisG2Selector.h"


namespace AxisG2Selector_ns
{

TFSM::TFSM(AxisG2Selector *dev){
	_status	= "";
	_dev = dev;
}

TFSM::~TFSM(){
}

bool  TFSM::addTcmds(string TCmds){
	#ifdef _DBG_TFSM_ADDCMDS
		cout << "TFSM::addTcmds(" << TCmds << ")" << endl;
	#endif /* _DBG_TFSM_ADDCMDS */

	vector<string> a, b;

	a = splitby(TCmds.c_str(), ';');
	if (a.size() < 2) return false; // transizione non definita oppure definita senza almeno un cmd

	#ifdef _DBG_TFSM_ADDCMDS
		cout << "TFSM::addTcmds() transition: " << a[0] << endl;
	#endif /* _DBG_TFSM_ADDCMDS */
	
	vector<cmd_source_t> vcmds;
	cmd_source_t cmd;
	vector<sourceDev2_t>::iterator sit;

	for (size_t k=1; k<a.size(); k++){
		#ifdef _DBG_TFSM_ADDCMDS
			cout << "TFSM::addTcmds() cmd[" << k-1 << "]: " << a[k] << endl;
		#endif /* _DBG_TFSM_ADDCMDS */

		b = splitby(a[k].c_str(), ':');
		if (b.size() != 3) return false; // errore di formato, non e' : nikname:cmd:value

		for (sit=_dev->sources.begin(); sit!=_dev->sources.end(); ++sit){
			if ((*sit).nickName.compare(b[0]) == 0){

				cmd.source_idx = (*sit).index;
				cmd.cmd     = b[1];
				if (b[2].empty()) cmd.cmd_val = NAN;
				else cmd.cmd_val = atof(b[2].c_str());

				vcmds.push_back(cmd);

				#ifdef _DBG_TFSM_ADDCMDS
					cout << "TFSM::addTcmds() " << b[0] << " with index " << cmd.source_idx << " ";
					cout << "cmd: " << cmd.cmd << " ";
					cout << "value: " << cmd.cmd_val;
					cout << endl;
				#endif /* _DBG_TFSM_ADDCMDS */
			}
		}
	}

	if (vcmds.size() > 0 ) {
		tcmds_t tcmd;
		tcmd.transition = a[0];
		tcmd.cmds = vcmds;

		_tcmds.push_back(tcmd);

		return true;
	};
	return false;
}

void  TFSM::dumpTcmds(){
	vector<tcmds_t>::iterator it;
	vector<cmd_source_t>::iterator cit;

	for (it=_tcmds.begin(); it!=_tcmds.end(); ++it){
		cout << "transition = " << (*it).transition << endl;
		for (cit=(*it).cmds.begin(); cit!=(*it).cmds.end(); ++cit){
			cout << "          ";
			cout << "source_idx: " << (*cit).source_idx << " ";
			cout << "cmd: " << (*cit).cmd << " ";
			cout << "val: " << (*cit).cmd_val;
			cout << endl;
		}
		
	}
}



vector<string> TFSM::str_getTcmds(){
	vector<tcmds_t>::iterator it;
	vector<cmd_source_t>::iterator cit;
	vector<string> out;
	stringstream ss;

	for (it=_tcmds.begin(); it!=_tcmds.end(); ++it){
		ss.str("");
		ss << "transition: " << (*it).transition << " | ";
		for (cit=(*it).cmds.begin(); cit!=(*it).cmds.end(); ++cit){
			ss << (_dev->sources[(*cit).source_idx]).dev << ":";
			//ss << (*cit).source_idx << ":";
			ss << (*cit).cmd_val;
			ss << " | ";
		}
		out.push_back(ss.str());
		
	}
	return out;
}


void  TFSM::tfsm(string new_status){           // proceed with FSM

	if (!_enable) return;

	#ifdef _DBG_TFSM_0
		cout << "TFSM: -- IN --- status : " << _status ;
		cout << " -- new_status : " << new_status << " ----" << endl;
	#endif /* _DBG_TFSM_0 */

	if (
		(_dev->fSMTransitionNoTargetEnable || (new_status.compare(_dev->noTargetValue) != 0)) &&
		(new_status.compare(_status) != 0) &&
		( _dev->fSMTransitionOnMoving || (_dev->state() != Tango::MOVING))
	) {
		#ifdef _DBG_TFSM
			cout << "TFSM: --------- STATUS CHANGED ! -------- ";
			cout << " from : " << _status ;
			cout << " to : " << new_status ;
			cout << " | devState : " << Tango::DevStateName[_dev->state()];
			cout << " ";
		#endif /* _DBG_TFSM */

		if ( !_status.empty() && !new_status.empty() ){
			string transition = _status + "," + new_status;
			#ifdef _DBG_TFSM
				cout << "--------- TRANSITION! : " << transition << " -------- ";
			#endif /* _DBG_TFSM */

			//---------------------------------------------------------------------
			//	TRANSITION !
			//
			vector<tcmds_t>::iterator it;
			vector<cmd_source_t>::iterator cit;

			for (it=_tcmds.begin(); it!=_tcmds.end(); ++it){
				if (transition.compare((*it).transition) == 0 ) {
					#ifdef _DBG_TFSM
						cout << " transition found! ";
					#endif /* _DBG_TFSM */

					for (cit=(*it).cmds.begin(); cit!=(*it).cmds.end(); ++cit){
						try {
							if ( isnan((*cit).cmd_val) ) {
								_dev->sources[(*cit).source_idx].dp->command_inout((*cit).cmd);
							} else {
								Tango::DeviceData dd;
								dd << (*cit).cmd_val;
								_dev->sources[(*cit).source_idx].dp->command_inout((*cit).cmd, dd);
							}
                               			} catch(Tango::DevFailed &e) {
                                        		cout << e.errors[0].desc << endl;
						} catch (...) {
							cout << "ERROR!" << endl;
						}
					}

					break;
				}
			}
			
			//---------------------------------------------------------------------
		}

		#ifdef _DBG_TFSM
			cout << endl;
		#endif /* _DBG_TFSM */

		_status = new_status;
	}

	#ifdef _DBG_TFSM_0
		cout << "TFSM: -- OUT -- status : " << _status ;
		cout << " -- new_status : " << new_status << " ----" << endl;
	#endif /* _DBG_TFSM_0 */
}


} //namespace
